#!/usr/bin/env python
import wx, os, com, tarfile, shutil
from dialogs import IODialog, OKDialog, CSDialog
from Projects import filehandling

def init_dplug(dplug):
    """Base Function to initialize dplug"""
    dplug.plugins.project = NewProject(dplug)
    dplug.add_to("lexer", "projects")
    dplug.add_to("page_close", "projects")
    dplug.add_to("page_closing", "projects")
    dplug.add_to("save", "projects")
    dplug.add_to("configsave", "projects")
    dplug.add_to("configinit", "projects")

def trigger(trig, dplug, value=None):
    """Base Function to handle dplug triggers"""
    if trig == "quit":
        if dplug.plugins.project.status == "Enabled":
            dplug.plugins.project.close_project(dplug.plugins.project.projname)
    elif trig == "lexer":
        if dplug.plugins.project.is_project(dplug.main_class.get_active_tab().name):
            _tabholder = dplug.plugins.project.get_active_project().tabholder
            dplug.main_class.get_active_tab().editor = _tabholder.get_active_tab().editor
            _tabholder.refresh_file()
    elif trig == "page_close":
        if dplug.plugins.project.is_project(dplug.main_class.get_active_tab().name):
            name = dplug.plugins.project.get_active_project().tabholder.get_active_tab().name
            dplug.plugins.project.get_active_project().config.set(name, "open", 0)
            dplug.plugins.project.get_active_project().tabholder.close_tab()
            return False
        return True
    elif trig == "page_closing":
        #You can close a project at all times except that if you opened the x-th tab in your project and if your project is the x-th tab in the global notebook
        # Note from the author:  It was late and my love for complexness grows when I'm tired,  this is the result I guess it works but you need to think hard to understand what actually happens :D *ENJOY*
        if dplug.plugins.project.is_project(dplug.main_class.get_active_tab().name):
            if dplug.main_class.tabs[value].name == dplug.plugins.project.get_active_project().name:
                print "The global tab equals click selection"
                try:
                    if dplug.plugins.project.get_active_project().tabholder.tabs[value].name == dplug.plugins.project.get_active_project().tabholder.get_active_tab().name:
                        #the local tab equals click selection :O
                        #deditor will chose the local tab over the global tab as it is easier to close the global faster
                        pass
                    else:
                        return True
                except IndexError:
                    #there is definitly clicked on the global tab as there are not that many tabs in the local tab
                    return True
            name = dplug.plugins.project.get_active_project().tabholder.get_active_tab().name
            dplug.plugins.project.get_active_project().config.set(name, "open", 0)
            dplug.plugins.project.get_active_project().tabholder.close_tab()
            return False #We don't want the normal approach to close a tab as we have a mini-editor inside the tab
        return True  #No project open, so CLOSE THAT SH*T PLEASE
    elif trig == "save": #Might be able to remove this if proper tab "acting" is done
        if dplug.plugins.project.is_project(dplug.main_class.get_active_tab().name):
            dplug.plugins.project.get_active_project().tabholder.save()
            #dplug.trigger_update("saved") ??? other plugin compatibility check
            return False
        return True
    elif trig == "configsave":
        dplug.plugins.project.config.Update()
    elif trig == "configinit":
        dplug.plugins.project.config.Init()

class NewProject():
    """Class to manage Projects"""
    def __init__(self,dplug):
        #dplug init
        self.dplug = dplug
        self.gui = dplug.main_class
        
        #project vars init
        self.config = Config(self.gui.confpan, -1, i = self.dplug)
        self.projects = []
        self.mode = "classic"
        self.status = "disabled"
        
        #project dir init
        self.projectsdir = os.path.join(self.dplug.plugindir, "Projects")
        if os.path.exists(self.projectsdir):
            pass
        else:
            os.mkdir(self.projectsdir)
            open(os.path.join(self.projectsdir, "projects.ded"), "w").close()        
        #project menu init
        self.projectsmenu = wx.Menu()
        newid = wx.NewId()
        openid = wx.NewId()
        self.projectsmenu.Append(newid, "New Project")
        self.projectsmenu.Append(openid, "Open Project")
        self.gui.menu.Append(self.projectsmenu, "Projects")
        self.gui.Bind(wx.EVT_MENU, lambda event: self.new_project(), id=newid)
        self.gui.Bind(wx.EVT_MENU, lambda event: self.select_project(), id=openid)
    
    def create_project(self, name):
        """Create a new project with the name "name\""""
        open(os.path.join(self.projectsdir, "%s.ded" % name), "w").close()
        return True
    
    def open_project(self, name):
        """Open a project with the name "name\""""
        project = Project(name, self.dplug)
        self.projects.append(project)
    
    def open_project_file(self, path):
        """Open a project file from "path\""""
        if self.IsDpf(path):
            self.open_project(path)
        elif not self.IsTpf(path):
            OKDialog("Project Importing", "This is not a correct Tpf file")
    
    def IsDpf(self, path):
        """Check whether the "path" is a Deditor Project File"""
        if os.path.basename(path)[-3::] == "ded":
            config = com.Config(path)
            try:
                if config.get("general", "type") == "DPF":
                    return True
                else:
                    return False
            except:
                print "!WARNING:  Invalid .DED File!"
                return False
        return False
    
    def IsTpf(self, path):
        """Check whether the "path" is a Tar Project File"""
        try:
            tar = tarfile.open(path)
        except tarfile.ReadError:
            print "O_o InVaLiD o_O"
        else:
            name = os.path.basename(path)
            tarpath = os.path.join(self.projectsdir, "taryard", name)
            tar.extractall(tarpath)
            tar.close()
            for x in os.listdir(tarpath):
                if self.IsDpf(os.path.join(tarpath, x)):
                    self.import_project(tarpath, x)
                    return True
        return False
    
    def import_project(self, path, dpf):
        """import the project @"path\""""
        dir_dialog = wx.DirDialog(self.gui, "Where to import the project?", os.path.expanduser("~"))
        if dir_dialog.ShowModal() == wx.ID_OK:
            new_path = dir_dialog.GetPath()
        else:
            return
        shutil.rmtree(new_path) #the wxPython DirDialog already creates the folder whereas copytree wants to create it.
        shutil.copytree(path, new_path)
        
        conf = com.Config(os.path.join(new_path, dpf))
        
        cprefix = conf.get("general", "cprefix")
        for file in conf.config.sections():
            if file != "general":
                tmp = conf.get(file, "path").split(cprefix)[1]
                conf.set(file, "path", os.path.join(new_path, tmp))
        conf.set("general", "cprefix", new_path)
        conf.save()
        
        shutil.move(os.path.join(new_path, dpf), self.projectsdir)
        shutil.rmtree(path)
        self.open_project_file(os.path.join(self.projectsdir, dpf))

    def select_project(self):
        """Open the FileDialog to select a .ded file as project"""
        filters = "Ded Files (*.ded; *.dpf)|*.ded; *.dpf | Tar files (*.tar; *.tar.gz)|*.tar; *.tar.gz"
        dlg = wx.FileDialog(self.gui, message="Select File", defaultDir=self.projectsdir, wildcard=filters, style=wx.OPEN|wx.FD_CHANGE_DIR)
        if dlg.ShowModal() == wx.ID_OK:
            self.open_project_file(dlg.GetPath())
        dlg.Destroy()
    
    def new_project(self):
        """Open a dialog to choose a project name"""
        name = IODialog("New Project", "Name of the new project?").GetValue()
        if self.create_project(name):
            self.open_project("%s.ded" % name)
    
    def get_active_project(self):
        """If a project is opened, return the project"""
        page = self.gui.get_active_tab()
        if self.is_project(page.name):
            return self.get_project(page.name)
        return False
    
    def is_project(self, name):
        """Check whether "name" is a project"""
        for project in self.projects:
            if project.name == name:
                return True
    
    def get_project(self, name):
        """Get the project with name "name\""""
        for project in self.projects:
            if project.name == name:
                return project
    
    def close_project(self):
        """If a project is open, close it"""
        project = self.get_active_project()
        if project:
            project.close()
        else:
            OKDialog("Project Closing", "You tried to close a project while you aren't editing a project")

class Project:
    """A Project Instance"""
    def __init__(self, path, dplug):
        #vars
        self.dplug = dplug
        self.gui = dplug.main_class
        self.name = os.path.basename(path)
        self.projectsdir = os.path.join(self.dplug.plugindir, "Projects")
        
        self.project_file = path
        self.config = com.Config(self.project_file)
        
        #Layout
        self.panel = wx.Panel(self.gui.notebook)
        self.left_panel = wx.Panel(self.panel)
        self.button_panel = wx.Panel(self.left_panel)
        
        self.notebook = wx.aui.AuiNotebook(self.panel, -1)
        bmp = wx.Image("/home/darragh/development/deditor/testzone/3.0/Plugins/Projects/add.png", wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.add_button = wx.BitmapButton(self.button_panel, id = wx.ID_ANY, bitmap = bmp, size = (40, 40))
        bmp = wx.Image("/home/darragh/development/deditor/testzone/3.0/Plugins/Projects/save.png", wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.save_button = wx.BitmapButton(self.button_panel, id = wx.ID_ANY, bitmap = bmp, size = (40, 40))
        bmp = wx.Image("/home/darragh/development/deditor/testzone/3.0/Plugins/Projects/trashcan.png", wx.BITMAP_TYPE_ANY).ConvertToBitmap()
        self.remove_button = wx.BitmapButton(self.button_panel, id = wx.ID_ANY, bitmap = bmp, size = (40, 40))
        self.file_list = wx.ListBox(self.left_panel)
        
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.left_sizer = wx.BoxSizer(wx.VERTICAL)
        self.button_sizer = wx.BoxSizer(wx.HORIZONTAL)
        
        self.button_sizer.Add(self.add_button, 0)
        self.button_sizer.Add(self.save_button, 0)
        self.button_sizer.Add(self.remove_button, 0)
        self.left_sizer.Add(self.button_panel, 0)
        self.left_sizer.Add(self.file_list, 1, wx.EXPAND)
        self.sizer.Add(self.left_panel, 0, wx.EXPAND)
        self.sizer.Add(self.notebook, 1, wx.EXPAND)
        
        self.button_panel.SetSizer(self.button_sizer)
        self.left_panel.SetSizer(self.left_sizer)
        self.panel.SetSizer(self.sizer)
        
        #extra vars depending on layout
        self.tabholder = filehandling.TabHolder(self.notebook)
        
        self.files = []
        files = self.config.config.sections()
        if len(files) == 0:
            self.tabholder.new_file()
        for file in files:
            if not file == "general":
                self.files.append(self.config.get(file, "path"))
        
        #Events
        self.gui.Bind(wx.EVT_BUTTON, lambda event: self.add_dialog(), self.add_button)
        self.gui.Bind(wx.EVT_BUTTON, lambda event: self.save(), self.save_button)
        self.gui.Bind(wx.EVT_BUTTON, lambda event: self.remove_dialog(), self.remove_button)
        self.gui.Bind(wx.EVT_LISTBOX_DCLICK, self.open_file, self.file_list)
        
        #Database Queries
        if not "_#Empty#_" in self.files: #else new_file()
            for file in self.files:
                self.file_list.Append(os.path.basename(file))
                if self.config.get(os.path.basename(file), "open") == "1":
                    self.tabholder.open_file(file)
                    lang = self.gui.get_proglang(file)
                    self.tabholder.lexer(lang)
        
        self.gui.add_tab(self.name, self.panel)
        self.gui.notebook.SetSelection(self.gui.notebook.GetPageCount()-1)
        self.gui.Layout()
        
        self.gui.get_active_tab().editor = self.tabholder.get_active_tab().editor
    
    def get_file_names(self):
        """get a list with only the file names, attention! doubles possible"""
        tmp = []
        for file in self.files:
            tmp.append(os.path.basename(file))
        return tmp
    
    def get_file_paths(self):
        """get a list with only the file paths, attention! no doubles"""
        return self.files
    
    def get_file_path(self, name):
        """get the filepath of a file with basename ... Attention! Doubles"""
        for file in self.get_files():
            if file[0] == name:
                return file[1]
    
    def get_files(self):
        """get a list with file paths and basenames, attention! no doubles"""
        tmp = []
        for file in self.files:
            tmp.append([os.path.basename(file), file])
        return tmp
    
    def add_dialog(self):
        """Dialog to add a file"""
        dlg = wx.FileDialog(None, message="Select File", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.add_file(dlg.GetPath())
        dlg.Destroy()
    
    def add_file(self, path):
        """Method to add a file with path "path\""""
        if path in self.get_file_paths():
            OKDialog("File Adding", "This file is already in this project")
            return
        self.files.append(path)
        self.file_list.Append(os.path.basename(path))
        
        self.config.add_section(os.path.basename(path))
        self.config.set(os.path.basename(path), "path", path)
        self.check_cprefix()
        
        on_add = self.dplug.plugin_config.get("project", "on_add")
        if on_add == "Open":
            self.open_file(None, path)
    
    def check_cprefix(self):
        """check whether the common prefix in the [general] tab is still accurate"""
        fls = []
        for file in self.get_file_paths():
            fls.append(file)
        cprefix = os.path.commonprefix(fls)
        if self.config.get("general", "cprefix") != cprefix:
            self.config.set("general", "cprefix", cprefix)
    
    def open_file(self, evt, path=None):
        """Open a file from the project"""
        if path == None:
            file = self.file_list.GetString(evt.GetSelection())
            file = self.get_file_path(file)
        else:
            file = path
        self.tabholder.open_file(file)
        lang = self.gui.get_proglang(file)
        self.tabholder.lexer(lang)
        
        name = os.path.basename(file)
        self.config.set(name, "open", 1)
    
    def remove_dialog(self):
        """Dialog to confirm file removal"""
        if CSDialog("File Removal", "Are you sure you want to remove this file from the project?").GetValue():
            file = self.file_list.GetSelection()
            file = self.file_list.GetString(file)
            self.remove_file(file)
    
    def remove_file(self, name):
        """Remove the file with name "name\""""
        self.files.remove(self.get_file_path(name)) #WARNING THIS WILL ACT ABNORMAL WHEN MORE FILES WITH SAME NAME ARE IN THE PROJECT!!!
        self.file_list.Clear()
        for file in self.files:
            if file == "_#Empty#_":
                pass
            self.file_list.Append(os.path.basename(file))
        
        #Remove from notebook
        self.config.remove_section(name)
        
        self.check_cprefix()
    
    def close(self):
        """close the project"""
        #Conf Save
        self.gui.notebook.close_tab()
    
    def delete(self):
        """delete the project"""
        pass
    
    def save(self):
        """save the project"""
        filters = "Tar files (*.tar; *.tar.gz)|*.tar; *.tar.gz | All (*.*)| *.*"
        save_dialog = wx.FileDialog (self.panel, "Choose a location for your file", os.path.expanduser("~"), "%s.tar.gz" % self.name, filters, style = wx.SAVE )
        if save_dialog.ShowModal() == wx.ID_OK:
            tarname = save_dialog.GetPath()
        else:
            return
        tar = tarfile.open(tarname, "w:gz")
        temp_os = os.getcwd()
        cprefix = os.path.commonprefix(self.get_file_paths())
        
        os.chdir(os.path.dirname(self.project_file))
        tar.add(os.path.basename(self.project_file))
        
        os.chdir(cprefix)
        for file in self.get_file_paths():
            tar.add(file.split(os.path.dirname(cprefix) + "/")[1])
        tar.close()
        os.chdir(temp_os)

class Config(wx.Panel):
    def __init__(self, parent, sth, i):
        self.dplug = i
        wx.Panel.__init__(self, parent, sth)
        self.sizer = wx.GridSizer(2, 2, 0, 0)
        
        self.sizer.Add(wx.StaticText(self, -1, "Open file when added"))
        self.onadded = wx.CheckBox(self)
        self.sizer.Add(self.onadded)
        self.SetSizer(self.sizer)
        
        self.button_name = "Projects"

        parent.plugin_add(self)
        
    def Init(self):
        if self.dplug.plugin_config.get("project", "on_add") == "Open":
            self.onadded.SetValue(1)
        else:
            self.onadded.SetValue(0)

    def Update(self):
        if self.onadded.GetValue() == 1: self.dplug.plugin_config.set("project", "on_add", "Open")
        else: self.dplug.plugin_config.set("project", "on_add", "Silence")